!
! This work (Common Community Physics Package Framework), identified by
! NOAA, NCAR, CU/CIRES, is free of known copyright restrictions and is
! placed in the public domain.
!
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
! IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
! FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
! THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
! IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
! CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


!>
!! @brief Auto-generated cap module for the CCPP suite
!!
!
module ccpp_MusicBox_suite_cap

   use ccpp_kinds
   implicit none
   private
   
   ! Suite interfaces
   
   character(len=16) :: ccpp_suite_state = 'uninitialized'
   
   public :: MusicBox_suite_initialize
   public :: MusicBox_suite_timestep_initial
   public :: MusicBox_suite_physics
   public :: MusicBox_suite_timestep_final
   public :: MusicBox_suite_finalize
   
   ! Private suite variables

contains
   
   subroutine MusicBox_suite_initialize(errflg, errmsg, photo_lev, TimeStart, TimeEnd, dt, nbox,  &
        nlev, ntimes, file_times, nSpecies, nkRxt, njRxt, nlevels)
   
      use prepare_chemistry,       only: prepare_chemistry_init
      use get_environ_cond,        only: get_environ_cond_init
      use tuv_photolysis,          only: tuv_photolysis_init
      use chemistry_driver_ros,    only: chemistry_driver_ros_init
      use k_rateConst,             only: k_rateConst_init
      use photolysis_interstitial, only: photolysis_interstitial_init
      use mass_quantities_util,    only: mass_quantities_util_init
      use const_props_mod,         only: const_props_type

      ! Dummy arguments
      integer,            intent(out)   :: errflg
      character(len=512), intent(out)   :: errmsg
      integer,            intent(in)    :: photo_lev
      real(kind_phys),    intent(in)    :: TimeStart
      real(kind_phys),    intent(in)    :: TimeEnd
      real(kind_phys),    intent(in)    :: dt
      integer,            intent(in)    :: nbox
      integer,            intent(in)    :: nlev
      integer,            intent(out)   :: ntimes
      real(kind_phys),    allocatable, intent(out) :: file_times(:)
      integer,            intent(out)   :: nSpecies
      integer,            intent(out)   :: nkRxt
      integer,            intent(out)   :: njRxt
      integer,            intent(in)    :: nlevels
      
      ! Local Variables
      real(kind_phys),    allocatable   :: temp_constant(:,:)
      type(const_props_type),allocatable   :: cnst_info(:)
      real(kind_phys),    allocatable   :: o3(:,:)
      character(len=80)                 :: model_name

      if (trim(ccpp_suite_state) /= 'uninitialized') then
         errflg = 1
         write(errmsg, '(3a)') "Invalid initial CCPP state, '", trim(ccpp_suite_state),           &
              "' in MusicBox_suite_initialize"
         return
      end if
      allocate(temp_constant(1:nbox, 1:nlev))
      allocate(cnst_info(1:nSpecies))
      allocate(o3(1:nbox, 1:nlev))
      call k_rateConst_init(errmsg, errflg)
      call photolysis_interstitial_init(photo_lev, errmsg, errflg)
      call chemistry_driver_ros_init(TimeStart, TimeEnd, dt, errmsg, errflg)
      call get_environ_cond_init(nbox, nlev, ntimes, o3(:, :), temp_constant(:, :), file_times,   &
           errmsg, errflg)
      call prepare_chemistry_init(cnst_info, model_name, nSpecies, nkRxt, njRxt, errmsg, errflg)
      call mass_quantities_util_init(cnst_info, nSpecies, errmsg, errflg)
      call tuv_photolysis_init(nlevels, errmsg, errflg)
      deallocate(temp_constant)
      deallocate(cnst_info)
      deallocate(o3)
      ccpp_suite_state = 'initialized'
   
   end subroutine MusicBox_suite_initialize
   
   
   subroutine MusicBox_suite_timestep_initial(errflg, errmsg)
   

      ! Dummy arguments
      integer,            intent(out)   :: errflg
      character(len=512), intent(out)   :: errmsg

      if (trim(ccpp_suite_state) /= 'initialized') then
         errflg = 1
         write(errmsg, '(3a)') "Invalid initial CCPP state, '", trim(ccpp_suite_state),           &
              "' in MusicBox_suite_timestep_initial"
         return
      end if
      ccpp_suite_state = 'in_time_step'
   
   end subroutine MusicBox_suite_timestep_initial
   
   
   subroutine MusicBox_suite_physics(errflg, errmsg, press, temperature, vmr, density, mbar,      &
        nkRxt, njRxt, rh, c_h2o, zenith, albedo, press_mid, press_int, alt, temp, o2vmr, o3vmr,   &
        so2vmr, no2vmr, prates, o3totcol, TimeStart, TimeEnd)
   
      use chemistry_driver_ros,    only: chemistry_driver_ros_run
      use get_environ_cond,        only: get_environ_cond_run
      use prepare_chemistry,       only: prepare_chemistry_run
      use k_rateConst,             only: k_rateConst_run
      use tuv_photolysis,          only: tuv_photolysis_run
      use photolysis_interstitial, only: photolysis_interstitial_run
      use mass_quantities_util,    only: mass_quantities_util_run

      ! Dummy arguments
      integer,            intent(out)   :: errflg
      character(len=512), intent(out)   :: errmsg
      real(kind_phys),    intent(in)    :: press
      real(kind_phys),    intent(in)    :: temperature
      real(kind_phys),    intent(inout) :: vmr(:)
      real(kind_phys),    intent(out)   :: density
      real(kind_phys),    intent(out)   :: mbar
      integer,            intent(in)    :: nkRxt
      integer,            intent(in)    :: njRxt
      real(kind_phys),    intent(in)    :: rh
      real(kind_phys),    intent(in)    :: c_h2o
      real(kind_phys),    intent(in)    :: zenith
      real(kind_phys),    intent(in)    :: albedo
      real(kind_phys),    intent(in)    :: press_mid(:)
      real(kind_phys),    intent(in)    :: press_int(:)
      real(kind_phys),    intent(in)    :: alt(:)
      real(kind_phys),    intent(in)    :: temp(:)
      real(kind_phys),    intent(in)    :: o2vmr(:)
      real(kind_phys),    intent(in)    :: o3vmr(:)
      real(kind_phys),    intent(in)    :: so2vmr(:)
      real(kind_phys),    intent(in)    :: no2vmr(:)
      real(kind_phys),    intent(out)   :: prates(:,:)
      real(kind_phys),    intent(out)   :: o3totcol
      real(kind_phys),    intent(in)    :: TimeStart
      real(kind_phys),    intent(in)    :: TimeEnd
      
      ! Local Variables
      real(kind_phys),    allocatable   :: k_rateConst(:)
      real(kind_phys),    allocatable   :: j_rateConst(:)

      if (trim(ccpp_suite_state) /= 'in_time_step') then
         errflg = 1
         write(errmsg, '(3a)') "Invalid initial CCPP state, '", trim(ccpp_suite_state),           &
              "' in MusicBox_suite_physics"
         return
      end if
      allocate(k_rateConst(1:nkRxt))
      allocate(j_rateConst(1:njRxt))
      call prepare_chemistry_run(errmsg, errflg)
      call get_environ_cond_run(errmsg, errflg)
      call mass_quantities_util_run(press, temperature, vmr, density, mbar, errmsg, errflg)
      call k_rateConst_run(nkRxt, njRxt, k_rateConst, density, rh, c_h2o, temperature, errmsg,    &
           errflg)
      call tuv_photolysis_run(zenith, albedo, press_mid, press_int, alt, temp, o2vmr, o3vmr,      &
           so2vmr, no2vmr, prates(:, :), o3totcol, errmsg, errflg)
      call photolysis_interstitial_run(prates(:, :), j_rateConst, errmsg, errflg)
      call chemistry_driver_ros_run(vmr, TimeStart, TimeEnd, j_rateConst, k_rateConst, density,   &
           errmsg, errflg)
      deallocate(k_rateConst)
      deallocate(j_rateConst)
      ccpp_suite_state = 'in_time_step'
   
   end subroutine MusicBox_suite_physics
   
   
   subroutine MusicBox_suite_timestep_final(errflg, errmsg)
   

      ! Dummy arguments
      integer,            intent(out)   :: errflg
      character(len=512), intent(out)   :: errmsg

      if (trim(ccpp_suite_state) /= 'in_time_step') then
         errflg = 1
         write(errmsg, '(3a)') "Invalid initial CCPP state, '", trim(ccpp_suite_state),           &
              "' in MusicBox_suite_timestep_final"
         return
      end if
      ccpp_suite_state = 'initialized'
   
   end subroutine MusicBox_suite_timestep_final
   
   
   subroutine MusicBox_suite_finalize(errflg, errmsg)
   
      use get_environ_cond,        only: get_environ_cond_finalize
      use tuv_photolysis,          only: tuv_photolysis_finalize
      use photolysis_interstitial, only: photolysis_interstitial_finalize
      use mass_quantities_util,    only: mass_quantities_util_finalize
      use chemistry_driver_ros,    only: chemistry_driver_ros_finalize

      ! Dummy arguments
      integer,            intent(out)   :: errflg
      character(len=512), intent(out)   :: errmsg

      if (trim(ccpp_suite_state) /= 'initialized') then
         errflg = 1
         write(errmsg, '(3a)') "Invalid initial CCPP state, '", trim(ccpp_suite_state),           &
              "' in MusicBox_suite_finalize"
         return
      end if
      call photolysis_interstitial_finalize(errmsg, errflg)
      call chemistry_driver_ros_finalize(errmsg, errflg)
      call get_environ_cond_finalize(errmsg, errflg)
      call mass_quantities_util_finalize(errmsg, errflg)
      call tuv_photolysis_finalize(errmsg, errflg)
      ccpp_suite_state = 'uninitialized'
   
   end subroutine MusicBox_suite_finalize
   

end module ccpp_MusicBox_suite_cap

