Hello World
===========

**HelloWorld** is an introduction to use of the CCPP Framework to
connect a supporting host model with CCPP-compliant physics schemes
and suites. In this example, the simple host model is represented by:
 * **hello_world_host.F90**: The main program and a subroutine which
 calls a physics suite.
 * **hello_world_mod.F90**: A module with host-model variables usable
 by the CCPP framework. The module also contains code to initialize
 the host state and to check the results of running the example
 physics suite.

**HelloWorld** comes with a single suite defined in the file,
  **hello_world_suite.xml**. The suite consists of two 'physics'
  schemes, **hello_scheme** (found in **hello_scheme.F90**) and
  **temp_adjust** (found in **temp_adjust.F90**).

Each Fortran (`.F90`) file above is paired with a corresponding
`.meta` file (e.g., **hello_world_mod.meta** and
**hello_world_mod.F90**). The `.meta` file contains CCPP metadata
which documents the data, datatypes, and/or schemes contained in the
accompanying Fortran (`.F90`) file.

To build and run the HelloWorld example using CMake, follow these steps:

```
cd <ccpp_framework>/doc/HelloWorld
mkdir build && cd build
cmake .. && make
./HelloWorld
```

The `cmake` step performs the following steps:
 1. Run `ccpp_capgen.py` to generate CCPP framework interface files
 2. Generate a dependency graph and create a Makefile

The `make` step compiles the source files and the files generated by
the `ccpp_capgen.py` and creates the `HelloWorld` executable.

 1. The `ccpp_capgen.py` step looks like:
```
../../../scripts/ccpp_capgen.py                                       \
        --host-files ../hello_world_host.meta,../hello_world_mod.meta \
        --scheme-files ../hello_scheme.meta,../temp_adjust.meta       \
        --suites ../hello_world_suite.xml --generate-host-cap         \
        --host-name HelloWorld
```
This produces the following files:
 * **HelloWorld_ccpp_cap.F90**: The CCPP interface functions to be called by
the host model.
 * **ccpp_hello_world_suite_cap.F90**: The suite file which contains the
subroutines which execute each phase of the `hello_world_suite` physics
suite. In general, there is one cap file per suite definition file.
 * **ccpp_kinds.F90**: A module which defines common types such as
`kind_phys` which are used by both host models and CCPP physics schemes.
 * **capfiles.txt**: A file which lists all of the files generated by
the call to `ccpp_capgen.py` (except for capfiles.txt). This file can
be used in `cmake` (by including it in your CMakeLists.txt) or `make`
(by including it in your Makefile) as shown in CMakeLists.txt and
Makefile.example in this directory.

To build and run the HelloWorld example with just a Makefile (no
CMake), follow these steps:

```
cd <ccpp_framework>/doc/HelloWorld
mkdir build && cd build
make -f ../Makefile.example
./HelloWorld
```
The `make` step performs the following steps:
 1. Run `ccpp_capgen.py` to generate CCPP framework interface files
 2. Compile source files and the files generated by the
 `ccpp_capgen.py` and create the `HelloWorld` executable.

The `ccpp_capgen.py` step is the same as in the CMake example.
